class ResizeWithPadding(object):
  def __init__(self, size):
    self.size = size
  def __call__(self, img):
    w, h = img.size
    aspect_ratio = w / h
    if aspect_ratio > 1:
      new_w = self.size
      new_h = int(self.size / aspect_ratio)
    else:
      new_h = self.size
      new_w = int(self.size * aspect_ratio)
    resized_img = img.resize((new_w, new_h))
    new_img = Image.new("RGB", (self.size, self.size), color=(0, 0, 0))
    left = (self.size - new_w) // 2
    top = (self.size - new_h) // 2
    new_img.paste(resized_img, (left, top))
    return new_img




from datetime import datetime
import pandas as pd
import matplotlib.pyplot as plt
import seaborn as sb
import praw
import pandas as pd
from tqdm import tqdm
import requests
from PIL import Image
from torchvision import transforms
from io import BytesIO
import numpy as np
import seaborn as sns
from collections import Counter

import torch
from torch.utils.data import Dataset, DataLoader
import requests
from PIL import Image
from io import BytesIO
import numpy as np
from tqdm import tqdm
import torchvision.transforms as transforms
from torchinfo import summary

reddit = praw.Reddit(client_id = "",
            client_secret = "",
            username = '',
            password = '',
            redirect_uri = "",
            user_agent = "",
            check_for_async=False)

from PIL import UnidentifiedImageError

class InvalidSubreddit(Exception):
    pass
class RestrictedSubreddit(Exception):
    pass

class ScrapeditDataset(Dataset):
  def __init__(self, subreddit: list, limit=10, sortby='year', show_safe=None, max_size=500, transform=None):
    if transform: self.transform = transform
    else: self.transform = transforms.ToTensor()
    self.subreddit = subreddit
    self.max_size = max_size
    self.results = {}
    self.limit = limit
    print(f"Collecting Data from {len(self.subreddit)} subreddits")
    for r in self.subreddit:
      sub = reddit.subreddit(r)
      try:
        sub_type = sub.subreddit_type
        if sub_type != 'public':
          raise RestrictedSubreddit(f"r/{r} is restricted to public access")
      except: raise InvalidSubreddit(f"r/{r} is invalid Subreddit, make sure the subreddit is valid")
      sub_itter = sub.top(sortby, limit=self.limit)
      self.result = []
      for submission in tqdm(sub_itter, total = self.limit):
        self.d = {}
        try: 
          if show_safe == True and submission.over_18 == True: break
          elif show_safe == False and submission.over_18 == False: break
          self.d['image'] = submission.preview["images"][0]["source"]["url"]
        except: self.d['image'] = None
        self.d['domain'] = submission.domain
        self.result.append(self.d)
      self.results[r] = self.result

    self.colated_ds = []
    print(f"Collecting Images from {len(self.subreddit)} subreddits")
    for r in self.subreddit:
      for image in tqdm(self.results[r], total = len(self.results)) :
        try:
          response = requests.get(image['image'])
          pil_image = Image.open(BytesIO(response.content))
          self.colated_ds.append([pil_image, r])
        except: pass

    z = [i[1] for i in self.colated_ds]
    self.ds_count = dict(Counter(z))

  def __call__(self):
    print(f"Out of {len(self.subreddit) * self.limit}, f{len(self.colated_ds)} were able to scrape")
    plt.bar(self.ds_count.keys(), self.ds_count.values())
    plt.title('Distribution of Data Sources')
    plt.xlabel('Data Sources')
    plt.ylabel('Count')
    plt.xticks(rotation=45, ha='right')
    plt.grid(axis='y', linestyle='--', alpha=0.7)
    plt.tight_layout()
    plt.show()

  def __len__(self):
    return len(self.colated_ds)

  def __getitem__(self, idx):
    pil_image, y = self.colated_ds[idx]
    try:
      pil_image.thumbnail((self.max_size, self.max_size), Image.LANCZOS)
      if pil_image.mode != 'RGB': pil_image = pil_image.convert('RGB')
      img_tensor = self.transform(pil_image)
      if img_tensor.shape[0] == 3: img_tensor = img_tensor.permute(1, 2, 0)
      np_image = np.array(img_tensor)
      return {'image': np_image, 'y': y}
    except UnidentifiedImageError:
      return None



def show_images(links_, title = None, figsize=(15,15), sub_title=None, noframe=True, max_col = 6, max_size=500, max_images = None, fontsize = 10, **kwargs):
  transform = transforms.ToTensor()
  if isinstance(links_, str): links = [links_]
  if max_images: num_images = min(len(links_), max_images)
  else: num_images = len(links_)
  num_rows = (num_images - 1) // max_col + 1
  fig, axes = plt.subplots(num_rows, max_col, figsize=figsize)
  if num_images > 1: axes = axes.flatten()
  if sub_title:
    if isinstance(sub_title, list) and len(sub_title) == num_images: pass
    elif isinstance(sub_title, str): sub_title = [sub_title] * num_images
    else: sub_title = [''] * num_images
  failed = 0
  links = []
  for link in links_:
    if link is not None: 
      links.append(link)
    else: failed+=1
  if max_images and len(links)<= max_images:
    for link in links_:
      if link not in links_: links.append(link)
      if len(links) == max_images: break
  links = links[:num_images]
  print(f"Displaying {len(links)} images in total")
  for i, link in enumerate(links):
    try: 
      response = requests.get(link)
      image = Image.open(BytesIO(response.content))
      image.thumbnail((max_size, max_size), Image.LANCZOS)
      if image.mode != 'RGB': image = image.convert('RGB')
      img_tensor = transform(image)
      if img_tensor.shape[0] == 3: img_tensor = img_tensor.permute(1, 2, 0)
      np_image = np.array(img_tensor)
      if sub_title:
        word = sub_title[i] if len(sub_title[i]) <= 45 else ""
        axes[i].set_title(word, fontsize = fontsize)
      axes[i].imshow(np_image, **kwargs)
    except: pass
    if noframe: axes[i].axis('off')
  for j in range(i + 1, num_rows * max_col):
      axes[j].axis('off')
  if title: plt.suptitle(title)
  print(f"{failed} post did not have images, or failed to fetch")
  plt.tight_layout()
  plt.show()


def show_batch(sample_batch, max = 25, X = 'image', y = 'y', cmap=None, **kwargs):
  if max:
    images = sample_batch[X][:max]
    labels = sample_batch[y][:max]
  else:
    images = sample_batch[X]
    labels = sample_batch[y]

  img_len = images.shape[0]
  row = int(math.sqrt(img_len))
  img_len = row ** 2

  fig, axes = plt.subplots(row, row, figsize=(25,25))
  axes = axes.flatten()

  for i in range(img_len):
    axes[i].imshow(images[i], **kwargs)
    axes[i].set_title(labels[i])
    axes[i].axis('off')



import tqdm
import praw
from datetime import datetime
import pandas as pd

class InvalidSubreddit(Exception):
    pass
class RestrictedSubreddit(Exception):
    pass
class AuthFailed(Exception):
   pass
class IncompleteAuth(Exception): pass
   
auths = []

def auth_reddit(client_id, client_secret, username, password, redirect_uri, user_agent, check_for_async = False):
  reddit = praw.Reddit(client_id = client_id,
            client_secret = client_secret,
            username = username,
            password = password,
            redirect_uri = redirect_uri,
            user_agent = user_agent,
            check_for_async=check_for_async)
  sub = reddit.subreddit('aww')
  try: sub_type = sub.subreddit_type
  except: raise InvalidSubreddit("Invalid Authentication, please recheck and try again")
  print("Authentication was successful")
  auths.append(reddit)

def scrape_reddit(subreddit: str, limit = 10, sortby = 'year', show_safe = None):
  if len(auths) == 0: raise IncompleteAuth("Complete Authentication by calling `scrapedit.auth_reddit` before proceeding")
  reddit = auths[0]
  sub = reddit.subreddit(subreddit)
  try:
    sub_type = sub.subreddit_type
    if sub_type != 'public':
      raise RestrictedSubreddit(f"r/{subreddit} is restricted to public access")
  except: raise InvalidSubreddit(f"r/{subreddit} is invalid Subreddit, make sure the subreddit is valid")
  result = []
  sub_itter = sub.top(sortby,limit = limit)
  for submission in tqdm(sub_itter):
    d = {}
    d['id'] = submission.id
    d['title'] = submission.title
    d['num_comments'] = submission.num_comments
    d['score'] = submission.score
    d['upvote_ratio'] = submission.upvote_ratio
    d['date'] = datetime.fromtimestamp(submission.created_utc)
    d['domain'] = submission.domain
    d['nsfw'] = submission.over_18
    try: d['image'] = submission.preview["images"][0]["source"]["url"]
    except: d['image'] = None
    try: d['author'] = submission.author.name
    except: d['author'] = 'Not Found'
    if show_safe == True and d['nsfw'] == True: d={}
    if show_safe == False and d['nsfw'] == False: d={}
    result.append(d)
  result = [item for item in result if item]
  return pd.DataFrame(result)


class ScrapeditDataset(Dataset):
  def __init__(self, subreddit: list, limit=10, sortby='year', show_safe=None, max_size=500, transform=None):
    if transform: self.transform = transform
    else: self.transform = transforms.ToTensor()
    self.subreddit = subreddit
    self.max_size = max_size
    self.results = {}
    for r in self.subreddit:
      sub = reddit.subreddit(r)
      sub_itter = sub.top(sortby, limit=limit)
      self.result = []
      for submission in tqdm(sub_itter):
        self.d = {}
        try:
            self.d['image'] = submission.preview["images"][0]["source"]["url"]
        except:
            self.d['image'] = None
        self.d['domain'] = submission.domain
        self.result.append(self.d)
      self.results[r] = self.result

    self.colated_ds = []
    for r in self.subreddit:
      for image in self.results[r]:
        try:
          response = requests.get(image['image'])
          self.colated_ds.append([image['image'], r])
        except: pass

  def __len__(self):
    return len(self.colated_ds)

  def __getitem__(self, idx):

    link, y = self.colated_ds[idx]
    response = requests.get(link)
    image = Image.open(BytesIO(response.content))
    image.thumbnail((self.max_size, self.max_size), Image.LANCZOS)
    if image.mode != 'RGB': image = image.convert('RGB')
    img_tensor = self.transform(image)
    if img_tensor.shape[0] == 3: img_tensor = img_tensor.permute(1, 2, 0)
    np_image = np.array(img_tensor)
    return {'image': np_image, 'y': y}



from setuptools import setup, find_packages
from os import path
working_directory = path.abspath(path.dirname(__file__))

with open(path.join(working_directory, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()

setup(
    name='scrapeddit', # name of packe which will be package dir below project
    version='0.1.4',
    url='https://github.com/Mafaz03/ReditScraper',
    author='Mafaz03',
    author_email='mohdmafaz200303@gmail.com',
    description='Simple test package',
    long_description=long_description,
    long_description_content_type='text/markdown',
    packages=find_packages(),
    install_requires=['praw', 'torchinfo'],
)